/*
  Java gfx-test framework

   Copyright (C) 2013  Red Hat

This file is part of IcedTea.

IcedTea is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

IcedTea is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with IcedTea; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
*/

package org.gfxtest.framework;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.image.BufferedImage;
import java.awt.image.BufferedImageOp;



/**
 * This class contains methods which performs various BitBlt operations.
 * 
 * @author Pavel Tisnovsky
 */
public class BitBltOperations
{
    /**
     * Create buffered bitmap with given type.
     * 
     * @param type
     *            type of the created image
     * @return new buffered bitmap
     */
    protected static BufferedImage createBufferedBitmap(int type)
    {
        return new BufferedImage(CommonBitmapOperations.DEFAULT_TEST_IMAGE_WIDTH, CommonBitmapOperations.DEFAULT_TEST_IMAGE_HEIGHT, type);
    }

    /**
     * Performs basic BitBlt operation for given source and destination image.
     * No mirroring, scaling and/or clipping are performed during BitBlt.
     * 
     * @param sourceImage
     *            source image for BitBlt
     * @param destinationImage
     *            destination image for BitBlt graphics canvas
     * @param graphics2d
     *            instance of class used during rendering
     * @return if the image has completely loaded and its pixels are no longer
     *         being changed, then returns true. Otherwise, returns false
     */
    protected static boolean performBitBlt(BufferedImage sourceImage, TestImage destinationImage, Graphics2D graphics2d)
    {
        // compute top-left corner coordinates
        final int x = (destinationImage.getWidth() - CommonBitmapOperations.DEFAULT_TEST_IMAGE_WIDTH) >> 1;
        final int y = (destinationImage.getHeight() - CommonBitmapOperations.DEFAULT_TEST_IMAGE_HEIGHT) >> 1;
        return graphics2d.drawImage(sourceImage, x, y, null);
    }

    /**
     * Performs basic BitBlt operation for given source and destination image.
     * No mirroring, scaling and/or clipping are performed during BitBlt.
     * 
     * @param sourceImage
     *            source image for BitBlt
     * @param destinationImage
     *            destination image for BitBlt graphics canvas
     * @param graphics2d
     *            instance of class used during rendering
     * @param bgcolor
     *            background color
     * @return if the image has completely loaded and its pixels are no longer
     *         being changed, then returns true. Otherwise, returns false
     */
    protected static boolean performBitBlt(BufferedImage sourceImage, TestImage destinationImage, Graphics2D graphics2d, Color bgcolor)
    {
        // compute top-left corner coordinates
        final int x = (destinationImage.getWidth() - CommonBitmapOperations.DEFAULT_TEST_IMAGE_WIDTH) >> 1;
        final int y = (destinationImage.getHeight() - CommonBitmapOperations.DEFAULT_TEST_IMAGE_HEIGHT) >> 1;
        return graphics2d.drawImage(sourceImage, x, y, bgcolor, null);
    }

    /**
     * Performs BitBlt operation for given source and destination image. Scaling
     * are performed according to parameters width and height.
     * 
     * @param sourceImage
     *            source image for BitBlt
     * @param destinationImage
     *            destination image for BitBlt graphics canvas
     * @param graphics2d
     *            instance of class used during rendering
     * @param width
     *            width of the destination region
     * @param height
     *            height of the destination region
     * @return if the image has completely loaded and its pixels are no longer
     *         being changed, then returns true. Otherwise, returns false
     */
    protected static boolean performBitBlt(BufferedImage sourceImage, TestImage destinationImage, Graphics2D graphics2d, int width, int height)
    {
        // compute top-left corner coordinates
        final int x = (destinationImage.getWidth() - width) >> 1;
        final int y = (destinationImage.getHeight() - height) >> 1;
        return graphics2d.drawImage(sourceImage, x, y, width, height, null);
    }

    /**
     * Performs BitBlt operation for given source and destination image. Scaling
     * are performed according to parameters width and height.
     * 
     * @param sourceImage
     *            source image for BitBlt
     * @param destinationImage
     *            destination image for BitBlt graphics canvas
     * @param graphics2d
     *            instance of class used during rendering
     * @param bgcolor
     *            background color
     * @param width
     *            width of the destination region
     * @param height
     *            height of the destination region
     * @return if the image has completely loaded and its pixels are no longer
     *         being changed, then returns true. Otherwise, returns false
     */
    protected static boolean performBitBlt(BufferedImage sourceImage, TestImage destinationImage, Graphics2D graphics2d, Color bgcolor, int width, int height)
    {
        // compute top-left corner coordinates
        final int x = (destinationImage.getWidth() - width) >> 1;
        final int y = (destinationImage.getHeight() - height) >> 1;
        return graphics2d.drawImage(sourceImage, x, y, width, height, bgcolor, null);
    }

    /**
     * Performs BitBlt operation for given source and destination image.
     * Horizontal and/or vertical flipping/mirroring are performed according to
     * parameters horizontalFlip and verticalFlip.
     * 
     * @param sourceImage
     *            source image for BitBlt
     * @param destinationImage
     *            destination image for BitBlt graphics canvas
     * @param graphics2d
     *            instance of class used during rendering
     * @param horizontalFlip
     *            enables performing horizontal flip of image
     * @param verticalFlip
     *            enables performing vertical flip of image
     * @return if the image has completely loaded and its pixels are no longer
     *         being changed, then returns true. Otherwise, returns false
     */
    protected static boolean performBitBlt(BufferedImage sourceImage, TestImage destinationImage, Graphics2D graphics2d,
                    boolean horizontalFlip, boolean verticalFlip)
    {
        // compute top-left corner coordinates of destination image
        final int dx1 = (destinationImage.getWidth() - CommonBitmapOperations.DEFAULT_TEST_IMAGE_WIDTH) >> 1;
        final int dy1 = (destinationImage.getHeight() - CommonBitmapOperations.DEFAULT_TEST_IMAGE_HEIGHT) >> 1;
        // compute bottom-right corner coordinates of destination image
        final int dx2 = dx1 + CommonBitmapOperations.DEFAULT_TEST_IMAGE_WIDTH;
        final int dy2 = dy1 + CommonBitmapOperations.DEFAULT_TEST_IMAGE_HEIGHT;
        // compute top-left corner coordinates of source image
        final int sx1 = horizontalFlip ? CommonBitmapOperations.DEFAULT_TEST_IMAGE_WIDTH: 0;
        final int sy1 = verticalFlip ? CommonBitmapOperations.DEFAULT_TEST_IMAGE_HEIGHT : 0;
        // compute bottom-right corner coordinates of source image
        final int sx2 = horizontalFlip ? 0 : CommonBitmapOperations.DEFAULT_TEST_IMAGE_WIDTH;
        final int sy2 = verticalFlip ? 0 : CommonBitmapOperations.DEFAULT_TEST_IMAGE_HEIGHT;
        return graphics2d.drawImage(sourceImage, dx1, dy1, dx2, dy2, sx1, sy1, sx2, sy2, null);
    }

    /**
     * Performs BitBlt operation for given source and destination image.
     * Horizontal and/or vertical flipping/mirroring are performed according to
     * parameters horizontalFlip and verticalFlip.
     * 
     * @param sourceImage
     *            source image for BitBlt
     * @param destinationImage
     *            destination image for BitBlt graphics canvas
     * @param graphics2d
     *            instance of class used during rendering
     * @param bgcolor
     *            background color
     * @param horizontalFlip
     *            enables performing horizontal flip of image
     * @param verticalFlip
     *            enables performing vertical flip of image
     * @return if the image has completely loaded and its pixels are no longer
     *         being changed, then returns true. Otherwise, returns false
     */
    protected static boolean performBitBlt(BufferedImage sourceImage, TestImage destinationImage, Graphics2D graphics2d,
                    Color bgcolor, boolean horizontalFlip, boolean verticalFlip)
    {
        // compute top-left corner coordinates of destination image
        final int dx1 = (destinationImage.getWidth() - CommonBitmapOperations.DEFAULT_TEST_IMAGE_WIDTH) >> 1;
        final int dy1 = (destinationImage.getHeight() - CommonBitmapOperations.DEFAULT_TEST_IMAGE_HEIGHT) >> 1;
        // compute bottom-right corner coordinates of destination image
        final int dx2 = dx1 + CommonBitmapOperations.DEFAULT_TEST_IMAGE_WIDTH;
        final int dy2 = dy1 + CommonBitmapOperations.DEFAULT_TEST_IMAGE_HEIGHT;
        // compute top-left corner coordinates of source image
        final int sx1 = horizontalFlip ? CommonBitmapOperations.DEFAULT_TEST_IMAGE_WIDTH: 0;
        final int sy1 = verticalFlip ? CommonBitmapOperations.DEFAULT_TEST_IMAGE_HEIGHT : 0;
        // compute bottom-right corner coordinates of source image
        final int sx2 = horizontalFlip ? 0 : CommonBitmapOperations.DEFAULT_TEST_IMAGE_WIDTH;
        final int sy2 = verticalFlip ? 0 : CommonBitmapOperations.DEFAULT_TEST_IMAGE_HEIGHT;
        return graphics2d.drawImage(sourceImage, dx1, dy1, dx2, dy2, sx1, sy1, sx2, sy2, bgcolor, null);
    }

    /**
     * Performs BitBlt operation for given source and destination image.
     * Cropping is used according to parameter cropRegion.
     *
     * @param sourceImage
     *            source image for BitBlt
     * @param destinationImage
     *            destination image for BitBlt graphics canvas
     * @param graphics2d
     *            instance of class used during rendering
     * @param cropRegion
     *            region used to crop part of the image
     * @return if the image has completely loaded and its pixels are no longer
     *         being changed, then returns true. Otherwise, returns false
     */
    protected static boolean performBitBlt(BufferedImage sourceImage, TestImage destinationImage, Graphics2D graphics2d,
                    Rectangle cropRegion)
    {
        // compute top-left corner coordinates of destination image
        final int dx1 = (destinationImage.getWidth() - CommonBitmapOperations.DEFAULT_TEST_IMAGE_WIDTH) >> 1;
        final int dy1 = (destinationImage.getHeight() - CommonBitmapOperations.DEFAULT_TEST_IMAGE_HEIGHT) >> 1;
        // compute bottom-right corner coordinates of destination image
        final int dx2 = dx1 + cropRegion.width;
        final int dy2 = dy1 + cropRegion.height;
        // compute top-left corner coordinates of source image
        final int sx1 = cropRegion.x;
        final int sy1 = cropRegion.y;
        // compute bottom-right corner coordinates of source image
        final int sx2 = sx1 + cropRegion.width;
        final int sy2 = sy1 + cropRegion.height;
        return graphics2d.drawImage(sourceImage, dx1, dy1, dx2, dy2, sx1, sy1, sx2, sy2, null);
    }

    /**
     * Performs BitBlt operation for given source and destination image.
     * Cropping is used according to parameter cropRegion.
     *
     * @param sourceImage
     *            source image for BitBlt
     * @param destinationImage
     *            destination image for BitBlt graphics canvas
     * @param graphics2d
     *            instance of class used during rendering
     * @param bgcolor
     *            background color
     * @param cropRegion
     *            region used to crop part of the image
     * @return if the image has completely loaded and its pixels are no longer
     *         being changed, then returns true. Otherwise, returns false
     */
    protected static boolean performBitBlt(BufferedImage sourceImage, TestImage destinationImage, Graphics2D graphics2d,
                    Color bgcolor, Rectangle cropRegion)
    {
        // compute top-left corner coordinates of destination image
        final int dx1 = (destinationImage.getWidth() - CommonBitmapOperations.DEFAULT_TEST_IMAGE_WIDTH) >> 1;
        final int dy1 = (destinationImage.getHeight() - CommonBitmapOperations.DEFAULT_TEST_IMAGE_HEIGHT) >> 1;
        // compute bottom-right corner coordinates of destination image
        final int dx2 = dx1 + cropRegion.width;
        final int dy2 = dy1 + cropRegion.height;
        // compute top-left corner coordinates of source image
        final int sx1 = cropRegion.x;
        final int sy1 = cropRegion.y;
        // compute bottom-right corner coordinates of source image
        final int sx2 = sx1 + cropRegion.width;
        final int sy2 = sy1 + cropRegion.height;
        return graphics2d.drawImage(sourceImage, dx1, dy1, dx2, dy2, sx1, sy1, sx2, sy2, bgcolor, null);
    }

    /**
     * Performs BitBlt operation for given source and destination image.
     * Selected raster operator is applied during BitBlt
     * 
     * @param sourceImage
     *            source image for BitBlt
     * @param destinationImage
     *            destination image for BitBlt graphics canvas
     * @param graphics2d
     *            instance of class used during rendering
     * @param rop
     *            raster operator to be applied during BibBlt
     * @return if the image has completely loaded and its pixels are no longer
     *         being changed, then returns true. Otherwise, returns false
     */
    protected static boolean performBitBlt(BufferedImage sourceImage, TestImage destinationImage, Graphics2D graphics2d,
                    BufferedImageOp rop)
    {
        // compute top-left corner coordinates
        final int x = (destinationImage.getWidth() - CommonBitmapOperations.DEFAULT_TEST_IMAGE_WIDTH) >> 1;
        final int y = (destinationImage.getHeight() - CommonBitmapOperations.DEFAULT_TEST_IMAGE_HEIGHT) >> 1;
        graphics2d.drawImage(sourceImage, rop, x, y);
        return true;
    }
}
