/*
  Java gfx-test framework

   Copyright (C) 2010, 2011  Red Hat

This file is part of IcedTea.

IcedTea is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

IcedTea is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with IcedTea; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
 */

package org.gfxtest.framework;

import java.awt.Graphics2D;
import java.awt.Polygon;
import java.awt.Rectangle;
import java.awt.geom.Area;
import java.awt.geom.CubicCurve2D;
import java.awt.geom.Line2D;
import java.awt.geom.Path2D;
import java.awt.geom.QuadCurve2D;



/**
 * This class contains method used for creating and rendering Areas created from
 * various types of shapes.
 * 
 * @author Pavel Tisnovsky
 */
public class CommonAreaOperations
{
    /**
     * Default Y offset of curve end points. Used during construction of
     * quadratic Bezier curves or cubic Bezier curves.
     */
    private static final int DEFAULT_Y_OFFSET = 40;

    /**
     * Offset from image borders used for rendering various graphics entities,
     * for example rectangles.
     */
    private static final int OFFSET = GfxTest.OFFSET;

    /**
     * Draw cross (mark) at specified coordinate
     * 
     * @param graphics
     *            graphics canvas
     * @param x
     *            x coordinate of a cross
     * @param y
     *            y coordinate of a cross
     */
    private static void drawCross(Graphics2D graphics2d, int x, int y)
    {
        GfxTest.drawCross(graphics2d, x, y);
    }

    /**
     * Draw crosses (marks) at the both end points of a line.
     * 
     * @param image
     *            image to which line is to be drawn
     * @param graphics2d
     *            graphics canvas
     */
    public static void drawCrossesAtLineControlPoints(TestImage image, Graphics2D graphics2d)
    {
        // calculate image dimensions
        int width =  image.getWidth();
        int height =  image.getHeight();

        // calculate line end points
        int x1 = GfxTest.OFFSET;
        int y1 = GfxTest.OFFSET;
        int x2 = width - GfxTest.OFFSET;
        int y2 = height - GfxTest.OFFSET;

        // draw marks at both end points
        drawCross(graphics2d, x1, y1);
        drawCross(graphics2d, x2, y2);

        // draw mark at the line center
        drawCross(graphics2d, x1 + x2 >> 1, y1 + y2 >> 1);
    }

    /**
     * Draw crosses (marks) at all four corners of a rectangle.
     * 
     * @param image
     *            image to which rectangle is to be drawn
     * @param graphics2d
     *            graphics canvas
     */
    public static void drawCrossesAtRectangleControlPoints(TestImage image, Graphics2D graphics2d)
    {
        // calculate image dimensions
        int width =  image.getWidth();
        int height =  image.getHeight();

        // calculate rectangle vertexes
        int x1 = OFFSET;
        int y1 = OFFSET;
        int x2 = width - OFFSET;
        int y2 = height - OFFSET;

        // draw marks at all four vertexes
        drawCross(graphics2d, x1, y1);
        drawCross(graphics2d, x2, y2);
        drawCross(graphics2d, x1, y2);
        drawCross(graphics2d, x2, y1);

        // draw mark in a rectangle center
        drawCross(graphics2d, x1 + x2 >> 1, y1 + y2 >> 1);
    }

    /**
     * Draw crosses (marks) at all vertexes of polygon.
     * 
     * @param image
     *            image to which polygon is to be drawn
     * @param graphics
     *            graphics canvas
     */
    public static void drawCrossesAtPolygonControlPoints(TestImage image, Graphics2D graphics2d)
    {
        // we already have implemented the requested functionality
        drawCrossesAtCubicCurveControlPoints(image, graphics2d);
    }

    /**
     * Draw crosses at the both end points and control point of quadratic Bezier
     * curve. Coordinates of curve control points are computed inside this
     * method.
     * 
     * @param image
     *            image to which quadratic curve and its control points are to be drawn
     * @param graphics2d
     *            graphics canvas
     */
    public static void drawCrossesAtQuadraticCurveControlPoints(TestImage image, Graphics2D graphics2d)
    {
        // calculate image dimensions
        int width = image.getWidth();
        int height = image.getHeight();

        // get curve coordinates
        int x1 = computeX1forQuadraticCurve(width);
        int y1 = computeY1forQuadraticCurve();
        int x2 = computeX2forQuadraticCurve(width);
        int y2 = computeY2forQuadraticCurve(height);
        int x3 = computeX3forQuadraticCurve(width);
        int y3 = computeY3forQuadraticCurve();

        // draw all three crosses at the coordinates of control points
        drawCross(graphics2d, x1, y1);
        drawCross(graphics2d, x2, y2);
        drawCross(graphics2d, x3, y3);
    }

    /**
     * Draw crosses at the both end points and both control point of cubic
     * Bezier curve. Coordinates of curve control points are computed inside
     * this method.
     * 
     * @param image
     *            image to which cubic curve and its control points are to be drawn
     * @param graphics2d
     *            graphics canvas
     */
    public static void drawCrossesAtCubicCurveControlPoints(TestImage image, Graphics2D graphics2d)
    {
        // construct point set which consists of all four curve control points
        CubicCurvePointSet pointSet = new CubicCurvePointSet(image);

        // fill in arrays containing coordinates for all cubic curve control points
        int[] x = pointSet.getXPointArray();
        int[] y = pointSet.getYPointArray();

        // draw all four crosses at the coordinates of control points
        for (int i = 0; i < 4; i++)
        {
            drawCross(graphics2d, x[i], y[i]);
        }
    }

    /**
     * Create area from simple Line2D.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return area created from simple Line2D.
     */
    public static Area createAreaFromLine2D(TestImage image)
    {
        // calculate image dimensions
        int width =  image.getWidth();
        int height =  image.getHeight();

        // get line end points
        int x1 = OFFSET;
        int y1 = OFFSET;
        int x2 = width - OFFSET;
        int y2 = height - OFFSET;

        // create line
        Line2D line = new Line2D.Double(x1, y1, x2, y2);

        // and return the area created from this line
        return new Area(line);
    }

    /**
     * Create area from Rectangle.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return area created from Rectangle.
     */
    public static Area createAreaFromRectangle(TestImage image)
    {
        // calculate image dimensions
        int width =  image.getWidth();
        int height =  image.getHeight();

        // get one vertex of a rectangle
        int x1 = OFFSET;
        int y1 = OFFSET;

        // calculate rectangle size
        int rectangleWidth = width - OFFSET * 2;
        int rectangleHeight = height - OFFSET * 2;

        // create rectangle
        Rectangle rectangle = new Rectangle(x1, y1, rectangleWidth, rectangleHeight);

        // and return the area created from this rectangle
        return new Area(rectangle);
    }

    /**
     * Create area from Polygon.
     *
     * @param image
     *            image to which area is to be drawn
     * @return area created from Polygon.
     */
    public static Area createAreaFromPolygon(TestImage image)
    {
        // construct point set which consists of all four curve control points
        CubicCurvePointSet pointSet = new CubicCurvePointSet(image);

        // fill in arrays containing path coordinates
        int[] x = pointSet.getXPointArray();
        int[] y = pointSet.getYPointArray();

        // create polygon
        Polygon polygon = new Polygon(x, y, x.length);

        // and return the area created from this polygon
        return new Area(polygon);
    }

    /**
     * Create area from the open quadratic Bezier curve.
     *
     * @param image
     *            image to which area is to be drawn
     * @return area created from open quadratic Bezier curve.
     */
    public static Area createAreaFromQuadraticCurve(TestImage image)
    {
        // calculate image dimensions
        int width =  image.getWidth();
        int height =  image.getHeight();

        // get curve coordinates
        int x1 = computeX1forQuadraticCurve(width);
        int y1 = computeY1forQuadraticCurve();
        int x2 = computeX2forQuadraticCurve(width);
        int y2 = computeY2forQuadraticCurve(height);
        int x3 = computeX3forQuadraticCurve(width);
        int y3 = computeY3forQuadraticCurve();

        // create open quadratic curve
        QuadCurve2D curve = new QuadCurve2D.Double(x1, y1, x2, y2, x3, y3);

        // and return the area created from this curve
        return new Area(curve);
    }

    /**
     * Create area from the open cubic Bezier curve.
     *
     * @param image
     *            image to which area is to be drawn
     * @return area created from open cubic Bezier curve.
     */
    public static Area createAreaFromCubicCurve(TestImage image)
    {
        // construct point set which consists of all four curve control points
        CubicCurvePointSet pointSet = new CubicCurvePointSet(image);

        // fill in arrays containing coordinates for all cubic curve control points
        int[] x = pointSet.getXPointArray();
        int[] y = pointSet.getYPointArray();

        // create open cubic curve
        CubicCurve2D curve = new CubicCurve2D.Double();
        // set control points coordinates
        curve.setCurve(x[0], y[0], x[1], y[1], x[2], y[2], x[3], y[3]);

        // and return the area created from this curve
        return new Area(curve);
    }

    /**
     * Create area from the open Path constructed from line segments
     *
     * @param image
     *            image to which area is to be drawn
     * @return area created from open Path
     */
    public static Area createAreaFromOpenPath(TestImage image)
    {
        // construct point set which consists of all four curve control points
        CubicCurvePointSet pointSet = new CubicCurvePointSet(image);

        // fill in arrays containing path coordinates
        int[] x = pointSet.getXPointArray();
        int[] y = pointSet.getYPointArray();

        // create Path2D
        Path2D path = new Path2D.Double();
        path.moveTo(x[0], y[0]);
        path.lineTo(x[1], y[1]);
        path.lineTo(x[2], y[2]);
        path.lineTo(x[3], y[3]);

        // and return the area created from this curve
        return new Area(path);
    }

    /**
     * Create area from the closed Path constructed from line segments.
     * 
     * @param image
     *            image to which area is to be drawn
     * @return area created from closed Path
     */
    public static Area createAreaFromClosedPath(TestImage image)
    {
        // construct point set which consists of all four curve control points
        CubicCurvePointSet pointSet = new CubicCurvePointSet(image);

        // fill in arrays containing path coordinates
        int[] x = pointSet.getXPointArray();
        int[] y = pointSet.getYPointArray();

        // create Path2D
        Path2D path = new Path2D.Double();
        path.moveTo(x[0], y[0]);
        path.lineTo(x[1], y[1]);
        path.lineTo(x[2], y[2]);
        path.lineTo(x[3], y[3]);
        path.closePath();

        // and return the area created from this curve
        return new Area(path);
    }

    /**
     * Compute X coordinate of quadratic Bezier curve first end point.
     * 
     * @param width
     *            canvas width
     * @return X coordinate of quadratic Bezier curve first end point.
     */
    private static int computeX1forQuadraticCurve(int width)
    {
        return width >> 2;
    }

    /**
     * Compute X coordinate of quadratic Bezier curve control point.
     * 
     * @param width
     *            canvas width
     * @return X coordinate of quadratic Bezier curve control point.
     */
    private static int computeX2forQuadraticCurve(int width)
    {
        return width >> 1;
    }

    /**
     * Compute X coordinate of quadratic Bezier curve second end point.
     * 
     * @param width
     *            canvas width
     * @return X coordinate of quadratic Bezier curve second end point.
     */
    private static int computeX3forQuadraticCurve(int width)
    {
        return width * 3 / 4;
    }

    /**
     * Compute Y coordinate of quadratic Bezier curve first end point.
     *
     * @return Y coordinate of quadratic Bezier curve first end point.
     */
    private static int computeY1forQuadraticCurve()
    {
        return DEFAULT_Y_OFFSET;
    }

    /**
     * Compute Y coordinate of quadratic Bezier curve control point.
     *
     * @param height
     *            canvas height
     * @return Y coordinate of quadratic Bezier curve control point.
     */
    private static int computeY2forQuadraticCurve(int height)
    {
        return height - DEFAULT_Y_OFFSET;
    }

    /**
     * Compute Y coordinate of quadratic Bezier curve second end point.
     *
     * @return Y coordinate of quadratic Bezier curve second end point.
     */
    private static int computeY3forQuadraticCurve()
    {
        return DEFAULT_Y_OFFSET;
    }

}
